<template>
    <el-dialog title="导出预览" top="7vh" width="77%" :visible.sync="dialogTableVisible" class="preview-dialog"
        destroy-on-close :before-close="close" :append-to-body="true" v-if="dialogTableVisible">
        <div :class="[load ? 'load-div' : '']" v-loading="load" element-loading-text="拼命加载中"
            element-loading-spinner="el-icon-loading" element-loading-background="rgba(7, 45, 98, 1)">
            <el-checkbox :indeterminate="indeterminate" @change="handleCheckAllChange">全选</el-checkbox>
            <div class="dialog-box">
                <div v-for="(item, index) in indexData" :key="index" style="transform: scale(1);">
                    <el-checkbox v-model="item.export">{{ item.name }}</el-checkbox>
                    <div class="index-item">
                        <div class="sub-title-content">
                            <span>
                                <img class="levelPageTitle2" src="@/assets/img/bigScreen/levelPageTitle2.png" />
                                <img class="air-title" src="@/assets/img/bigScreen/air-title.png" />
                            </span>
                            {{ item.title }}
                        </div>
                        <mapTarget @mounted="domToImg" :indexItem="item" :CQId="`cq-contianer_${index + 1}`"
                            :main9Id="`main9-contianer_${index + 1}`"></mapTarget>
                    </div>
                </div>
            </div>
        </div>

        <div slot="footer" class="dialog-footer">
            <el-button v-if="!load" type="primary" @click="downloadImg" :loading="downloadLoading">确认导出</el-button>
        </div>
    </el-dialog>
</template>

<script>
import html2canvas from 'html2canvas'
import mapTarget from '@/views/BigScreen/components/subComponents/mapTarget.vue' // 重点关注指标
import JSZip from 'jszip'
export default {
    components: {
        mapTarget
    },
    data() {
        return {
            load: false,
            downloadLoading: false,
            dialogTableVisible: false,
            indexData: [],
            indeterminate: true,
            doms: null,
            len: 0
        }
    },
    mounted() {

    },
    methods: {
        open(lsIndex, totalGear) {
            this.load = true
            let arr = [totalGear, ...lsIndex]
            arr.forEach(item => {
                item.export = true
                item.dataURL = ''
            })
            this.indexData = JSON.parse(JSON.stringify(arr))
            this.dialogTableVisible = true
            this.$nextTick(() => {
                this.getScale()
                this.doms = document.querySelectorAll('.index-item')
            })
        },
        getScale() {
            this.$nextTick(() => {
                let dialogBody = document.querySelector('.el-dialog__body')
                const wh = dialogBody.clientHeight / (935 * 3)
                const ww = dialogBody.clientWidth / (1919 * 3)
                let bl = ww < wh ? ww : wh
                let doms = document.querySelectorAll('.index-item')
                Array.from(doms).map(item => {
                    item.style.transform = `scale(${bl})`
                    item.style.transformOrigin = 'left top'
                })
            })
        },
        handleCheckAllChange(e) {
            this.indexData.forEach(item => item.export = e)
            this.indeterminate = !e
        },
        // 将dom转化为图片
        async domToImg() {
            this.len++
            if (this.len === this.indexData.length) {
                const processNext = (index) => {
                    if (index < this.indexData.length) {
                        this.getImgUrl(index).then(() => {
                            processNext(index + 1);
                        })
                    } else {
                        this.load = false;
                    }
                };
                processNext(0);
            } else {
                return;
            }
        },
        getImgUrl(index) {
            return new Promise(resolve => {
                html2canvas(this.doms[index], {
                    allowTaint: true,
                    scale: 1,
                    backgroundColor: "#072d62",
                    display: 'black',
                    useCORS: true,
                    allowTaint: true,
                    logging: false,
                    removeContainer: true,
                    width: 1920,
                    height: 935,
                    onclone: (e) => {
                        let doms = e.querySelectorAll('.index-item')
                        doms[index].style.transform = "none"
                    }

                }).then((canvas) => {
                    const dataURL = canvas.toDataURL('image/png');
                    this.indexData[index].dataURL = dataURL
                    resolve()
                })
            })
        },
        close() {
            this.indexData = []
            this.len = 0
            this.doms = null
            Object.assign(this.$data, this.$options.data())
            this.dialogTableVisible = false
        },
        downloadImg() {
            this.downloadLoading = true
            // 根据目标宽度和高度将图片进行缩放
            function getResizedImageData(image, targetWidth, targetHeight) {
                let canvas = document.createElement('canvas')
                let context = canvas.getContext('2d')
                canvas.width = targetWidth
                canvas.height = targetHeight
                context.drawImage(image, 0, 0, targetWidth, targetHeight)
                return canvas.toDataURL('image/png').split(';base64,')[1]
            }
            let data = this.indexData.filter(item => item.export)
            let imgUrl = []
            let promises = data.map(item => {
                let img = new Image()
                img.src = item.dataURL
                return new Promise(resolve => {
                    img.onload = () => {
                        imgUrl.push({
                            name: item.title + '.png',
                            url: getResizedImageData(img, 1920, 935)
                        })
                        resolve()
                    }
                })
            })
            Promise.all(promises).then(() => {
                let zip = new JSZip()
                imgUrl.forEach(file => {
                    zip.file(file.name, file.url, { base64: true })
                })
                zip.generateAsync({ type: 'blob' }).then(content => {
                    saveAs(content, '改革报表指标晾晒.zip')
                    this.$message.success('导出文件成功')
                    this.downloadLoading = false
                })
            })
        }
    },
}
</script>

<style lang="scss" scoped>
.load-div {
    height: 100%;
    overflow: hidden;
}

/deep/.detail-btn {
    display: none;
}

.index-item {
    width: 1919PX;
    height: 935PX;
    padding: 0 20PX 43PX;
    box-sizing: border-box;
    position: fixed;
    top: 20px;
    left: 0;
    transform-origin: left top;
}

/deep/.el-dialog__footer,
/deep/.el-dialog__header {
    background: #072d62;
}

/deep/.el-dialog__header {
    background-color: #072d62;
    color: #fff;

    span {
        color: #fff;
    }
}

/deep/.el-dialog__headerbtn .el-dialog__close {
    color: #fff !important;
}

/deep/.el-dialog__body {
    height: 68vh;
    min-height: 68vh;
    overflow-x: auto;

    .el-checkbox {
        margin-left: 5px;
    }

    .el-checkbox__label {
        padding-left: 5px;
        color: #fff;
    }

    >* {
        letter-spacing: 2PX;
    }

    margin: 0;
    padding: 0 20px;
    background: #072d62;

    .dialog-box {
        width: 100%;
        display: flex;
        flex-wrap: wrap;
        justify-content: flex-start;
        // gap: 5PX;

        >div {
            width: calc((100% - 10PX) / 3);
            height: 250px;
        }

        :nth-child(3n) {
            margin-right: 0;
        }
    }

    .sub-title-content {
        width: 100%;
        font-size: 30PX;
        letter-spacing: 2PX;
        line-height: 60PX;
        color: #fff;
        position: relative;
        padding-left: 30PX;

        .levelPageTitle2 {
            width: 25PX;
            height: 40PX;
            position: absolute;
            top: 50%;
            transform: translateY(-50%);
            left: 5PX;
        }

        .air-title {
            width: 40PX;
            position: absolute;
            top: 50%;
            transform: translateY(-50%);
            left: -5PX;
        }
    }

    .map-tip-container {
        width: 868PX;
        margin-left: 30PX;

        .map-container {
            height: 520PX;
            margin-bottom: 28PX;
        }
    }

    .list-container {
        width: 927PX;

        >* {
            font-family: 'Microsoft YaHei', sans-serif;
        }
    }

    .sub-title-content::before {
        content: '';
        position: absolute;
        left: 10PX;
        top: 5PX;
        width: 50PX;
        height: 50PX;
        background-size: cover;
    }

    .sub-title-content::after {
        content: '';
        position: absolute;
        left: 10PX;
        top: 5PX;
        width: 50PX;
        height: 50PX;
        background-size: cover;
    }

    .map-target {
        padding: 8PX 0 0 0;
        width: 100%;
        height: 832PX;
        box-sizing: border-box;
        gap: 0;
    }

    .map-tip .tip-item .tip-text {
        font-size: 18PX;
    }

    .map-tip .tip-item .level-title {
        font-size: 18PX;
        line-height: 37PX;
    }
}

/deep/::-webkit-scrollbar {
    width: 0 !important;
}

/deep/ .map-tip {
    // bottom: 14PX;
    bottom: 16px;
}
</style>
